# fast-json-query

A lightweight and efficient JSON querying utility for filtering and searching data. Inspired by MongoDB's query syntax but designed for in-memory JSON data in Node.js.

[![npm version](https://badge.fury.io/js/fast-json-query.svg)](https://badge.fury.io/js/fast-json-query)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)

## Features

- 🚀 Lightweight and fast
- 💪 TypeScript support
- 🔍 MongoDB-like query syntax
- 📦 Zero dependencies
- 🌳 Nested object support
- ⚡ Rich query operators

## Installation

```bash
npm install fast-json-query
# or
yarn add fast-json-query
```

## Usage

```typescript
import queryJSON from 'fast-json-query';

const data = [
  { id: 1, name: 'Alice', age: 25, city: 'New York' },
  { id: 2, name: 'Bob', age: 30, city: 'London' },
  { id: 3, name: 'Charlie', age: 35, city: 'New York' },
];

// Simple equality
const newYorkers = queryJSON(data, { city: 'New York' });
// [{ id: 1, ... }, { id: 3, ... }]

// Comparison operators
const over30 = queryJSON(data, { age: { $gt: 30 } });
// [{ id: 3, ... }]

// Multiple conditions
const result = queryJSON(data, {
  city: 'New York',
  age: { $lt: 30 }
});
// [{ id: 1, ... }]
```

## Query Operators

| Operator | Description | Example |
|----------|-------------|---------|
| `$eq` | Equals | `{ age: { $eq: 25 } }` |
| `$ne` | Not equals | `{ age: { $ne: 25 } }` |
| `$gt` | Greater than | `{ age: { $gt: 25 } }` |
| `$gte` | Greater than or equal | `{ age: { $gte: 25 } }` |
| `$lt` | Less than | `{ age: { $lt: 25 } }` |
| `$lte` | Less than or equal | `{ age: { $lte: 25 } }` |
| `$in` | In array | `{ city: { $in: ['NY', 'LA'] } }` |
| `$nin` | Not in array | `{ city: { $nin: ['NY', 'LA'] } }` |
| `$regex` | Regular expression | `{ name: { $regex: /^A/ } }` |

## Advanced Usage

### Nested Objects

```typescript
const users = [
  { 
    id: 1,
    profile: {
      name: 'John',
      address: {
        city: 'New York'
      }
    }
  }
];

const result = queryJSON(users, {
  'profile.address.city': 'New York'
});
```

### Multiple Conditions

```typescript
const result = queryJSON(data, {
  age: { $gt: 25, $lt: 35 },
  city: { $in: ['New York', 'London'] },
  active: true
});
```

## TypeScript Support

The package includes TypeScript definitions and supports generic types:

```typescript
interface User {
  id: number;
  name: string;
  age: number;
}

const users: User[] = [...];
const result = queryJSON<User>(users, { age: { $gt: 25 } });
```

## Performance

The package is designed to be lightweight and fast. It uses native JavaScript array methods and avoids unnecessary iterations.

## Contributing

Contributions are welcome! Please feel free to submit a Pull Request.

## License

MIT © [Md Azadur] 